<?php

namespace Drupal\iframe_lazy_loading\Plugin\Filter;

use Drupal\Component\Utility\Html;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\filter\Attribute\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\filter\Plugin\FilterInterface;

/**
 * Filter that adds lazy loading to iframes.
 */
#[Filter(
  "iframe_lazy_loading",
  new TranslatableMarkup("Add lazy loading to iframes"),
  FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
)]
class LazyLoadingIframes extends FilterBase {

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode): FilterProcessResult {
    $dom = Html::load($text);

    foreach ($dom->getElementsByTagName('iframe') as $node) {
      assert($node instanceof \DOMElement);
      $node->setAttribute('loading', 'lazy');
    }

    return new FilterProcessResult(Html::serialize($dom));
  }

}
